from __future__ import unicode_literals

from django.core.management.base import BaseCommand
from django.conf import settings
from json import loads
from pagetree.models import Hierarchy
from restclient import GET
import os.path


class Command(BaseCommand):
    args = '<hierarchy name> (optional)'
    help = 'pull the pagetree data down from production'

    def handle(self, *args, **options):
        if not settings.DEBUG:
            print("this should never be run on production")
            return
        if not hasattr(settings, 'PROD_BASE_URL'):
            print("you must set PROD_BASE_URL")
            return
        print("fetching content from prod...")
        url = settings.PROD_BASE_URL + "_pagetree/export/"
        if args:
            url = url + "?hierarchy=" + args[0]
        d = loads(GET(url))
        print("removing old pagetree hierarchy...")
        Hierarchy.objects.all().delete()
        print("importing the new one...")
        Hierarchy.from_dict(d)

        if not hasattr(settings, 'PROD_MEDIA_BASE_URL'):
            print("in order to pull down uploaded files,")
            print("you must set PROD_MEDIA_BASE_URL")

        print("pulling down uploaded files...")
        base_len = len(settings.PROD_MEDIA_BASE_URL)
        for upload in d.get('resources', []):
            relative_path = upload[base_len:]
            relative_dir = os.path.join(*os.path.split(relative_path)[:-1])
            full_dir = os.path.join(settings.MEDIA_ROOT, relative_dir)
            try:
                os.makedirs(full_dir)
            except OSError:
                pass
            with open(os.path.join(settings.MEDIA_ROOT,
                                   relative_path), "w") as f:
                print("  writing %s to %s" % (upload, relative_path))
                f.write(GET(upload))
        print("done")



from __future__ import unicode_literals

from django.http import HttpResponse, HttpResponseRedirect
from django.utils.encoding import smart_text
from django.shortcuts import (
    get_object_or_404, render_to_response, render)
from django.template.context import RequestContext
from django.template.defaultfilters import slugify
from django.template.response import TemplateResponse
from json import dumps, loads
from pagetree.helpers import get_section_from_path
from pagetree.forms import MoveSectionForm
from pagetree.models import Section, PageBlock, Hierarchy, Version


def reorder_pageblocks(request, section_id, id_prefix="pageblock_id_"):
    if request.method != "POST":
        return HttpResponse("only use POST for this")
    section = get_object_or_404(Section, id=section_id)
    section.save_version(request.user, activity="reorder pageblocks")
    keys = list(request.GET)
    keys.sort(key=lambda x: int(x.split('_')[-1]))
    pageblocks = [int(request.GET[k]) for k in keys if k.startswith(id_prefix)]
    section.update_pageblocks_order(pageblocks)
    return HttpResponse("ok")


def reorder_section_children(request, section_id, id_prefix="section_id_"):
    if request.method != "POST":
        return HttpResponse("only use POST for this")
    section = get_object_or_404(Section, id=section_id)
    section.save_version(request.user, activity="reorder children")
    keys = list(request.GET)
    keys.sort(key=lambda x: int(x.split('_')[-1]))
    children = [int(request.GET[k]) for k in keys if k.startswith(id_prefix)]
    section.update_children_order(children)
    section.clear_tree_cache()
    return HttpResponse("ok")


def delete_pageblock(request, pageblock_id, success_url=None):
    block = get_object_or_404(PageBlock, id=pageblock_id)
    if request.method == "POST":
        section = block.section
        section.save_version(request.user,
                             activity="delete block [%s]" % smart_text(block))
        try:
            block.block().delete()
        except AttributeError:
            # if the model has been refactored, we sometimes
            # end up with 'stub' pageblocks floating around
            # that no longer have a block object associated
            # it's nice to still be able to delete them
            # without having to scrap the whole db and start over
            pass
        block.delete()
        if success_url is None:
            success_url = section.get_edit_url()
        return HttpResponseRedirect(success_url)
    return TemplateResponse(request, 'delete_confirm.html')


def export_pageblock_json(request, pageblock_id):
    block = get_object_or_404(PageBlock, id=pageblock_id)
    the_json = dumps(block.as_dict())
    r = HttpResponse(the_json, content_type="application/json")
    r['Content-Disposition'] = ('attachment; filename=pageblock_%d.json'
                                % int(pageblock_id))
    return r


def import_pageblock_json(request, pageblock_id):
    block = get_object_or_404(PageBlock, id=pageblock_id)
    block.section.save_version(
        request.user,
        activity="importing pageblock json [%s]" % smart_text(block))
    if request.method == "POST":
        if 'file' not in request.FILES:
            return HttpResponse("you must upload a json file")
        json = loads(request.FILES['file'].read())
        block.import_from_dict(json)
        return HttpResponseRedirect(block.section.get_edit_url())
    else:
        return render_to_response("import_json.html", dict(),
                                  context_instance=RequestContext(request))


def edit_pageblock(request, pageblock_id, success_url=None):
    block = get_object_or_404(PageBlock, id=pageblock_id)
    section = block.section
    section.save_version(
        request.user,
        activity="edit pageblock [%s]" % smart_text(block))
    block.edit(request.POST, request.FILES)
    if success_url is None:
        success_url = section.get_edit_url()
    return HttpResponseRedirect(success_url)


def edit_section(request, section_id, success_url=None):
    section = get_object_or_404(Section, id=section_id)
    section.save_version(request.user, activity="edit section")
    section.label = request.POST.get('label', '')
    section.slug = slugify(request.POST.get('slug', section.label))[:50]
    section.show_toc = request.POST.get('show_toc', False)
    section.deep_toc = request.POST.get('deep_toc', False)
    section.save()
    section.enforce_slug()
    section.save()
    section.clear_caches()
    if success_url is None:
        success_url = section.get_edit_url()
    return HttpResponseRedirect(success_url)


def delete_section(request, section_id, success_url=None):
    section = get_object_or_404(Section, id=section_id)
    if request.method == "POST":
        parent = section.get_parent()
        if parent:
            parent.save_version(
                request.user,
                activity="delete child section [%s]" % smart_text(section))
        section.delete()
        if success_url is None:
            if parent:
                parent.clear_tree_cache()
                success_url = parent.get_edit_url()
            else:
                success_url = "/"
        return HttpResponseRedirect(success_url)
    return TemplateResponse(request, 'delete_confirm.html')


def move_section(request, section_id, success_url=None):
    if request.method != "POST":
        return HttpResponse("only use POST for this")
    section = get_object_or_404(Section, id=section_id)
    section.save_version(request.user, "move section")

    form = MoveSectionForm(request.POST, instance=section)
    if form.is_valid():
        to_section = get_object_or_404(
            Section,
            id=form.cleaned_data['_ref_node_id'])
        position = form.cleaned_data['_position']
        section.move(to_section, position)
        parent = section.get_parent()
        if parent:
            section.clear_tree_cache()
        to_section.clear_tree_cache()
        if success_url is None:
            success_url = to_section.get_edit_url()
        return HttpResponseRedirect(success_url)
    return HttpResponse("could not move section")


def add_pageblock(request, section_id, success_url=None):
    section = get_object_or_404(Section, id=section_id)
    blocktype = request.POST.get('blocktype', '')
    section.save_version(user=request.user,
                         activity="add pageblock [%s]" % blocktype)
    # now we need to figure out which kind of pageblock to create
    for pb_class in section.available_pageblocks():
        if pb_class.display_name == blocktype:
            # a match
            block = pb_class.create(request)
            section.append_pageblock(
                label=request.POST.get('label', ''),
                css_extra=request.POST.get('css_extra', ''),
                content_object=block)
    if success_url is None:
        success_url = section.get_edit_url()
    return HttpResponseRedirect(success_url)


def add_child_section(request, section_id, success_url=None):
    section = get_object_or_404(Section, id=section_id)
    label = request.POST.get('label', 'unnamed') or 'unnamed'
    section.save_version(
        request.user,
        "add child section [%s]" % label)
    slug = slugify(request.POST.get('slug', label))[:50]
    section.append_child(label, slug)
    section.clear_tree_cache()
    if success_url is None:
        success_url = section.get_edit_url()
    return HttpResponseRedirect(success_url)


def create_tree_root(request):
    get_section_from_path("")  # creates a root if one doesn't exist
    return HttpResponseRedirect(request.META['HTTP_REFERER'])


def exporter(request):
    hierarchy_name = request.GET.get('hierarchy', 'main')
    h = get_object_or_404(Hierarchy, name=hierarchy_name)
    data = h.as_dict()
    resources = []
    protocol = "http"
    if request.is_secure():
        protocol = "https"
    url_base = protocol + "//" + request.get_host()
    for pb in PageBlock.objects.filter(section__hierarchy=h):
        block = pb.block()
        if hasattr(block, 'list_resources'):
            for r in block.list_resources():
                resources.append(url_base + r)
    data['resources'] = resources
    resp = HttpResponse(dumps(data))
    resp['Content-Type'] = 'application/json'
    return resp


def revert_to_version(request, version_id):
    v = get_object_or_404(Version, pk=version_id)
    if request.method == "POST":
        v.section.save_version(
            request.user,
            activity="reverting to previous version [%d]" % v.id)
        # clear all pageblocks on the section
        for pb in v.section.pageblock_set.all():
            pb.delete()
        # clear child sections (since they might get replaced)
        v.section.get_children().delete()
        # force the numchild to 0.
        # possible treebeard bug we are working around
        # basically, despite calling the treebeard queryset's .delete()
        # as recommended, child nodes get deleted, but the parent's
        # numchild never gets reset, so
        #   section.get_num_children() returns, say, 3
        # while
        #   section.get_children() returns []
        # this really seems like a bug in treebeard, possibly
        # transaction related

        v.section.numchild = 0
        v.section.save()
        v.section.from_dict(loads(v.data))
        v.section.clear_tree_cache()
        return HttpResponseRedirect(v.section.get_edit_url())
    else:
        return render(request, "revert_confirm.html", dict(version=v))



from __future__ import unicode_literals

import abc
from six import add_metaclass

from django.contrib.auth.models import User
from django.apps import apps
from django.contrib.contenttypes.models import ContentType


@add_metaclass(abc.ABCMeta)
class ReportableInterface(object):
    """ Abstract base class for a reportable interface that can be added to
        a given pageblock to support dynamic discovery of reporting data

        To use:
        1. implement a custom ReportColumn for your block/user data
        2. implement the report_metadata and report_values on your block
        3. register as part of the ReportableInterface
           ReportableInterface.register()
    """

    @abc.abstractmethod
    def report_metadata(self):
        """ Return an array of implemented ReportColumns
        """
        return

    @abc.abstractmethod
    def report_values(self):
        """ Return an array of implemented ReportColumns
        """
        return


@add_metaclass(abc.ABCMeta)
class ReportColumnInterface(object):

    @abc.abstractmethod
    def identifier(self):
        """ Unique column identifier, returns a spaceless, lower-case string
        """
        return

    @abc.abstractmethod
    def metadata(self):
        """ Return an array of metadata information about this column
            Used in the report's "key" file

            return [hierarchy id, itemIdentifier, category, itemType, itemText,
                answerIdentifier, answerText]
        """
        return

    @abc.abstractmethod
    def user_value(self, user):
        """ Return the user value associated with this column """
        return


class StandaloneReportColumn(ReportColumnInterface):
    """ A generic report column with no pageblock/hierarchy association
        Small pieces of information like username, user profile information or
        percent complete are good candidates for a StandaloneReportColumn.
        The user value specification is handled by a passed in value function
    """

    def __init__(self, name, group, value_type, description, value_func):
        self.name = name
        self.group = group
        self.value_type = value_type
        self.description = description
        self.value_func = value_func

    def identifier(self):
        return self.name

    def metadata(self):
        # hierarchy is n/a
        return ["", self.name, self.group, self.value_type, self.description]

    def user_value(self, user):
        return self.value_func(user)


class PagetreeReport(object):

    def get_reportable_content_types(self):
        types = []
        hierarchy = apps.get_model('pagetree', 'hierarchy')
        for block in hierarchy.available_pageblocks():
            if issubclass(block, ReportableInterface):
                types.append(ContentType.objects.get_for_model(block))

        return types

    def __init__(self):
        self.types = self.get_reportable_content_types()

    def users(self):
        return User.objects.all()

    def standalone_columns(self):
        """ Return an array of columns with no pagegblock/hierarchy association
            example:
            return [StandaloneReportColumn('username', 'profile', 'string',
                                           'Unique Username',
                                           lambda x: x.username)]

            empty array returned by default
        """
        return []

    def metadata_columns(self, hierarchies):
        """ Iterate pageblocks in order, collect metadata columns """
        columns = self.standalone_columns()
        for hierarchy in hierarchies:
            for section in hierarchy.get_root().get_descendants():
                for p in section.pageblock_set.filter(
                        content_type__in=self.types):
                    columns += p.block().report_metadata()
        return columns

    def value_columns(self, hierarchies):
        """ Iterate pageblocks in order, collect value columns """
        columns = self.standalone_columns()
        for hierarchy in hierarchies:
            for section in hierarchy.get_root().get_descendants():
                for p in section.pageblock_set.filter(
                        content_type__in=self.types):
                    columns += p.block().report_values()
        return columns

    def value_headers(self, columns):
        headers = []
        for column in columns:
            headers += [column.identifier()]
        return headers

    def metadata(self, hierarchies):
        """
            A "key" for all questions and answers in the system.
            * One row for short/long text questions
            * Multiple rows for single/multiple-choice questions.
            Each question/answer pair should get its own  row
        """
        yield ['hierarchy', 'itemIdentifier', 'exercise type',
               'itemType', 'itemText', 'answerIdentifier', 'answerText']
        yield ''

        for column in self.metadata_columns(hierarchies):
            yield column.metadata()

    def values(self, hierarchies):
        """
        All system results
        * One or more column for each question in system.
            ** 1 column for short/long text. label = itemIdentifier from key
            ** 1 column for single choice. label = itemIdentifier from key
            ** n columns for multiple choice: 1 column for each possible answer
               *** column labeled as itemIdentifer_answer.id

            * One row for each user in the system.
                1. username
                2 - n: answers
                    * short/long text. text value
                    * single choice. answer.id
                    * multiple choice.
                        ** answer id is listed in each question/answer
                        column the user selected
                    * Unanswered fields represented as an empty cell
        """
        columns = self.value_columns(hierarchies)

        yield self.value_headers(columns)

        for user in self.users():
            row = []
            for column in columns:
                row.append(column.user_value(user))
            yield row



from __future__ import unicode_literals

from django import forms
from treebeard.forms import movenodeform_factory
from pagetree.models import Hierarchy, Section


class CloneHierarchyForm(forms.ModelForm):
    class Meta:
        model = Hierarchy
        fields = ['name', 'base_url']

    class Media:
        css = {
            'all': ('pagetree/css/loading.css',)
        }
        js = ('pagetree/js/src/clone-loading.js',)

    def clean(self):
        cleaned_data = super(CloneHierarchyForm, self).clean()
        name = cleaned_data.get('name')
        base_url = cleaned_data.get('base_url')

        if Hierarchy.objects.filter(name=name).count() > 0:
            raise forms.ValidationError(
                'There\'s already a hierarchy with the name: {}'.format(
                    name))

        if Hierarchy.objects.filter(base_url=base_url).count() > 0:
            raise forms.ValidationError(
                'There\'s already a hierarchy with the base_url: {}'.format(
                    base_url))

        return cleaned_data


MoveSectionForm = movenodeform_factory(
    Section,
    exclude=('label', 'slug', 'hierarchy', 'show_toc', 'deep_toc')
)



from __future__ import unicode_literals

from django import template
register = template.Library()


@register.filter('hasrelated')
def hasrelated(obj):
    return len(obj._meta.get_all_related_objects()) > 0



from __future__ import unicode_literals

from pagetree.models import Hierarchy
import warnings


def get_hierarchy(name="main", base_url="/"):
    if isinstance(name, Hierarchy):
        # sometimes it makes things simpler in other areas
        # when the 'h' object being passed around may be
        # a string or may already be a Hierarchy.
        # so let's be cool about that
        return name
    return Hierarchy.objects.get_or_create(
        name=name,
        defaults=dict(base_url=base_url))[0]


def get_section_from_path(path, hierarchy="main",
                          hierarchy_name=None, hierarchy_base="/"):
    # 'hierarchy_name' and 'hierarchy_base' pair are preferred
    # interface, but 'hierarchy' is the more common old version
    # so we have to support that as well. If name/base are set
    # we treat that as a signal that the caller is using the
    # new interface, so we ignore 'hierarchy'
    if hierarchy_name is None:
        hierarchy_name = hierarchy
    h = get_hierarchy(name=hierarchy_name, base_url=hierarchy_base)
    return h.get_section_from_path(path)


def get_module(section):
    warnings.warn(
        (
            "pagetree.helpers.get_module is deprecated "
            "in favor of Section.get_module()"))
    return section.get_module()


def needs_submit(section):
    warnings.warn(
        (
            "pagetree.helpers.needs_submit is deprecated "
            "in favor of Section.needs_submit()"))
    return section.needs_submit()


def submitted(section, user):
    warnings.warn(
        (
            "pagetree.helpers.submitted is deprecated "
            "in favor of Section.submitted()"))
    return section.submitted(user)


def block_submitted(block, user):
    if user.is_anonymous():
        # anon can't have submitted a block
        return False
    if hasattr(block, 'needs_submit'):
        if block.needs_submit():
            try:
                s = block.unlocked(user)
                if not s:
                    return False
            except:
                pass
    return True



# flake8: noqa
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Hierarchy',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=256)),
                ('base_url', models.CharField(default=b'', max_length=256)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageBlock',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('ordinality', models.PositiveIntegerField(default=1)),
                ('label', models.CharField(max_length=256, null=True, blank=True)),
                ('css_extra', models.CharField(help_text=b'extra CSS classes (space separated)', max_length=256, null=True, blank=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('section', 'ordinality'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('path', models.CharField(unique=True, max_length=255)),
                ('depth', models.PositiveIntegerField()),
                ('numchild', models.PositiveIntegerField(default=0)),
                ('label', models.CharField(max_length=256)),
                ('slug', models.SlugField()),
                ('show_toc', models.BooleanField(default=False, help_text=b'list table of contents of immediate child sections (if applicable)')),
                ('deep_toc', models.BooleanField(default=False, help_text=b'include children of children in TOC. (this only makes sense if the above is checked)')),
                ('hierarchy', models.ForeignKey(to='pagetree.Hierarchy')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TestBlock',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('body', models.TextField(blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserLocation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('path', models.CharField(default=b'/', max_length=256)),
                ('hierarchy', models.ForeignKey(to='pagetree.Hierarchy')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserPageVisit',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('status', models.CharField(default=b'incomplete', max_length=256)),
                ('first_visit', models.DateTimeField(auto_now_add=True)),
                ('last_visit', models.DateTimeField(auto_now=True)),
                ('section', models.ForeignKey(to='pagetree.Section')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Version',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('saved_at', models.DateTimeField(auto_now_add=True)),
                ('activity', models.TextField(default=b'', null=True, blank=True)),
                ('data', models.TextField(default=b'', null=True, blank=True)),
                ('comment', models.TextField(default=b'', null=True, blank=True)),
                ('section', models.ForeignKey(to='pagetree.Section')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-saved_at'],
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='userpagevisit',
            unique_together=set([('user', 'section')]),
        ),
        migrations.AlterUniqueTogether(
            name='userlocation',
            unique_together=set([('user', 'hierarchy')]),
        ),
        migrations.AddField(
            model_name='pageblock',
            name='section',
            field=models.ForeignKey(to='pagetree.Section'),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.test import TestCase
from django.test.client import RequestFactory
from django.test.client import Client
from django.http import Http404
from django.contrib.auth.models import User
import unittest
from pagetree.models import Hierarchy
from pagetree.generic.views import has_responses, visit_root
from pagetree.generic.views import UserPageVisitor
from pagetree.generic.views import generic_view_page
from pagetree.generic.views import generic_instructor_page
from pagetree.generic.views import generic_edit_page


class Anon(object):
    def is_anonymous(self):
        return True


class NonAnon(object):
    def is_anonymous(self):
        return False


class MockSection(object):
    def __init__(self, needs_submit_value=False):
        self.needs_submit_called = 0
        self.user_pagevisit_called = 0
        self.needs_submit_value = needs_submit_value

    def needs_submit(self):
        self.needs_submit_called += 1
        return self.needs_submit_value

    def user_pagevisit(self, user, status):
        self.user_pagevisit_called += 1
        self.user = user
        self.status = status

    def get_uservisit(self, user):
        return None


class UserPageVisitorTest(unittest.TestCase):
    def test_visit_anonymous(self):
        s = MockSection()
        upv = UserPageVisitor(s, Anon())
        upv.visit()
        self.assertEqual(s.needs_submit_called, 0)

    def test_visit_not_status(self):
        s = MockSection(needs_submit_value=False)
        upv = UserPageVisitor(s, NonAnon())
        upv.visit(status=False)
        self.assertEqual(s.needs_submit_called, 1)
        self.assertEqual(s.user_pagevisit_called, 1)
        self.assertEqual(s.status, "complete")

    def test_visit_not_status_needs_submit(self):
        s = MockSection(needs_submit_value=True)
        upv = UserPageVisitor(s, NonAnon())
        upv.visit(status=False)
        self.assertEqual(s.needs_submit_called, 1)
        self.assertEqual(s.user_pagevisit_called, 1)
        self.assertEqual(s.status, "incomplete")

    def test_visit_status(self):
        s = MockSection()
        upv = UserPageVisitor(s, NonAnon())
        upv.visit(status=True)
        self.assertEqual(s.needs_submit_called, 0)
        self.assertEqual(s.user_pagevisit_called, 1)
        self.assertEqual(s.status, True)


class OneLevelDeepTest(TestCase):
    def setUp(self):
        self.h = Hierarchy.objects.create(name="main", base_url="")
        self.root = self.h.get_root()
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 1',
                'slug': 'section-1',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 2',
                'slug': 'section-2',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 3',
                'slug': 'section-3',
                'pageblocks': [],
                'children': [],
            })
        r = self.root.get_children()
        self.section1 = r[0]
        self.section2 = r[1]
        self.section3 = r[2]

    def tearDown(self):
        self.h.delete()

    def test_has_responses(self):
        self.assertFalse(has_responses(self.section1))

    def test_visit_root(self):
        r = visit_root(self.section1)
        self.assertEqual(r.status_code, 302)

    def test_visit_root_fallback(self):
        r = visit_root(self.section3)
        self.assertEqual(r.status_code, 302)


class GenericPageViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.h = Hierarchy.objects.create(name="main", base_url="")
        self.root = self.h.get_root()
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 1',
                'slug': 'section-1',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 2',
                'slug': 'section-2',
                'pageblocks': [
                    {'label': 'Welcome to your new Forest Site',
                     'css_extra': '',
                     'block_type': 'Test Block',
                     'body': 'You should now use the edit link to add content',
                     },
                ],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 3',
                'slug': 'section-3',
                'pageblocks': [],
                'children': [],
            })
        r = self.root.get_children()
        self.section1 = r[0]
        self.section2 = r[1]
        self.section3 = r[2]
        self.u = User.objects.create(username="test")
        self.u.set_password("test")
        self.u.save()

    def test_nonexistant(self):
        request = self.factory.get("/foo/")
        with self.assertRaises(Http404):
            generic_view_page(request, "/foo/")

    def test_root(self):
        request = self.factory.get("/")
        response = generic_view_page(request, "/")
        self.assertEqual(response.status_code, 302)

    def test_section1(self):
        request = self.factory.get("/section-1/")
        request.user = self.u
        response = generic_view_page(
            request, "section-1/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_section1_gated(self):
        request = self.factory.get("/section-1/")
        request.user = self.u
        response = generic_view_page(
            request, "section-1/",
            gated=True,
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_section2(self):
        request = self.factory.get("/section-2/")
        request.user = self.u
        response = generic_view_page(
            request, "section-2/",
            extra_context=dict(foo='bar'),
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_section2_gated(self):
        request = self.factory.get("/section-2/")
        request.user = self.u
        response = generic_view_page(
            request, "section-2/",
            gated=True,
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 302)

    def test_section1_post(self):
        request = self.factory.post("/section-1/", dict())
        request.user = self.u
        response = generic_view_page(
            request, "section-1/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 302)

    def test_section1_reset(self):
        request = self.factory.post("/section-1/", dict(action="reset"))
        request.user = self.u
        response = generic_view_page(
            request, "section-1/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 302)

    def test_section3_post(self):
        request = self.factory.post("/section-3/", dict())
        request.user = self.u
        response = generic_view_page(
            request, "section-3/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 302)

    def test_instructor_section1(self):
        request = self.factory.get("/instructor/section-1/")
        request.user = self.u
        response = generic_instructor_page(
            request, "section-1/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_edit_section1(self):
        request = self.factory.get("/edit/section-1/")
        request.user = self.u
        response = generic_edit_page(
            request, "section-1/",
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_instructor_section1_extra(self):
        request = self.factory.get("/instructor/section-1/")
        request.user = self.u
        response = generic_instructor_page(
            request, "section-1/",
            extra_context=dict(foo="bar"),
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)

    def test_edit_section1_extra(self):
        request = self.factory.get("/edit/section-1/")
        request.user = self.u
        response = generic_edit_page(
            request, "section-1/",
            extra_context=dict(foo="bar"),
            template="pagetree/test_page.html")
        self.assertEqual(response.status_code, 200)


class GenericPageViewURLConfTest(TestCase):
    def setUp(self):
        self.h = Hierarchy.objects.create(name="main", base_url="")
        self.root = self.h.get_root()
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 1',
                'slug': 'section-1',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 2',
                'slug': 'section-2',
                'pageblocks': [
                    {'label': 'Welcome to your new Forest Site',
                     'css_extra': '',
                     'block_type': 'Test Block',
                     'body': 'You should now use the edit link to add content',
                     },
                ],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 3',
                'slug': 'section-3',
                'pageblocks': [],
                'children': [],
            })
        r = self.root.get_children()
        self.section1 = r[0]
        self.section2 = r[1]
        self.section3 = r[2]
        self.u = User.objects.create(username="test")
        self.u.set_password("test")
        self.u.save()
        self.c = Client()
        self.c.login(username="test", password="test")

    def test_edit_section1(self):
        response = self.c.get("/pages/edit/section-1/")
        self.assertEqual(response.status_code, 200)

    def test_instructor_section1(self):
        response = self.c.get("/pages/instructor/section-1/")
        self.assertEqual(response.status_code, 200)

    def test_section1(self):
        response = self.c.get("/pages/section-1/")
        self.assertEqual(response.status_code, 200)


class GenericPageViewURLConfTestSecond(TestCase):
    """ make sure non default hierarchies also work """
    def setUp(self):
        self.h = Hierarchy.objects.create(name="two", base_url="/pages2")
        self.root = self.h.get_root()
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 1',
                'slug': 'sec-1',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 2',
                'slug': 'sec-2',
                'pageblocks': [
                    {'label': 'Welcome to your new Forest Site',
                     'css_extra': '',
                     'block_type': 'Test Block',
                     'body': 'You should now use the edit link to add content',
                     },
                ],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 3',
                'slug': 'sec-3',
                'pageblocks': [],
                'children': [],
            })
        r = self.root.get_children()
        self.section1 = r[0]
        self.section2 = r[1]
        self.section3 = r[2]
        self.u = User.objects.create(username="test")
        self.u.set_password("test")
        self.u.save()
        self.c = Client()
        self.c.login(username="test", password="test")

    def test_edit_section1(self):
        response = self.c.get("/pages2/edit/sec-1/")
        self.assertEqual(response.status_code, 200)

    def test_instructor_section1(self):
        response = self.c.get("/pages2/instructor/sec-1/")
        self.assertEqual(response.status_code, 200)

    def test_section1(self):
        response = self.c.get("/pages2/sec-1/")
        self.assertEqual(response.status_code, 200)



from __future__ import unicode_literals

from django.test import TestCase
from django.contrib.auth.models import User
from django.utils.encoding import smart_text
from django.test.client import Client
from pagetree.models import Hierarchy, PageBlock
from json import loads


class TestEditViews(TestCase):
    def setUp(self):
        self.h = Hierarchy.objects.create(name="main", base_url="")
        self.root = self.h.get_root()
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 1',
                'slug': 'section-1',
                'pageblocks': [],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 2',
                'slug': 'section-2',
                'pageblocks': [
                    {'label': 'Welcome to your new Forest Site',
                     'css_extra': 'the-css-class',
                     'block_type': 'Test Block',
                     'body': 'You should now use the edit link to add content',
                     },
                ],
                'children': [],
            })
        self.root.add_child_section_from_dict(
            {
                'label': 'Section 3',
                'slug': 'section-3',
                'pageblocks': [],
                'children': [],
            })
        r = self.root.get_children()
        self.section1 = r[0]
        self.section2 = r[1]
        self.section3 = r[2]
        self.u = User.objects.create(username="test")
        self.u.set_password("test")
        self.u.save()
        self.c = Client()
        self.c.login(username="test", password="test")

    def test_delete_section(self):
        response = self.c.post(
            "/pagetree/delete_section/%d/" % self.section3.id, dict())
        self.assertEqual(response.status_code, 302)

    def test_delete_section_get(self):
        response = self.c.get(
            "/pagetree/delete_section/%d/" % self.section3.id, dict())
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "<form")

    def test_edit_section(self):
        response = self.c.post(
            "/pagetree/edit_section/%d/" % self.section3.id,
            dict(label="new label"))
        self.assertEqual(response.status_code, 302)

    def test_add_child_section(self):
        response = self.c.post(
            "/pagetree/section/add/%d/" % self.section3.id,
            dict(label="new label"))
        self.assertEqual(response.status_code, 302)

    def test_move_section(self):
        response = self.c.post(
            "/pagetree/section/move/%d/" % self.section3.id,
            dict())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_text(response.content),
                         "could not move section")

    def test_move_section_get(self):
        response = self.c.get(
            "/pagetree/section/move/%d/" % self.section3.id,
            dict())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_text(response.content),
                         "only use POST for this")

    def test_add_pageblock(self):
        response = self.c.post(
            "/pagetree/pageblock/add/%d/" % self.section3.id,
            dict(blocktype="Test Block", body="some text"))
        self.assertEqual(response.status_code, 302)

    def test_reorder_section_children_empty(self):
        response = self.c.post(
            "/pagetree/reorder_section_children/%d/" % self.section3.id,
            dict())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_text(response.content), "ok")

    def test_reorder_section_children_get(self):
        response = self.c.get(
            "/pagetree/reorder_section_children/%d/" % self.section3.id,
            dict())
        self.assertEqual(smart_text(response.content),
                         "only use POST for this")

    def test_reorder_pageblocks_empty(self):
        response = self.c.post(
            "/pagetree/reorder_pageblocks/%d/" % self.section3.id,
            dict())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_text(response.content), "ok")

    def test_reorder_pageblocks_get(self):
        response = self.c.get(
            "/pagetree/reorder_pageblocks/%d/" % self.section3.id,
            dict())
        self.assertEqual(smart_text(response.content),
                         "only use POST for this")

    def test_delete_pageblock(self):
        p = PageBlock.objects.all()[0]
        response = self.c.post("/pagetree/delete_pageblock/%d/" % p.id, dict())
        self.assertEqual(response.status_code, 302)

    def test_pageblock_json_export(self):
        p = PageBlock.objects.all()[0]

        response = self.c.get("/pagetree/pageblock/jsonexport/%d/" % p.id,
                              dict())
        self.assertEqual(response.status_code, 200)

        the_json = loads(smart_text(response.content))
        self.assertEqual(the_json["body"],
                         "You should now use the edit link to add content")
        self.assertEquals(the_json["css_extra"], "the-css-class")
        self.assertEquals(the_json["block_type"], "Test Block")
        self.assertEquals(the_json["label"], "Welcome to your new Forest Site")

    def test_edit_pageblock(self):
        p = PageBlock.objects.all()[0]
        response = self.c.post("/pagetree/pageblock/edit/%d/" % p.id,
                               dict(body="new body text"))
        self.assertEqual(response.status_code, 302)

    def test_exporter(self):
        response = self.c.get("/pagetree/export/")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], "application/json")

    def test_create_tree_root(self):
        response = self.c.get("/pagetree/create_tree_root",
                              {}, HTTP_REFERER='http://foo/bar')
        self.assertEqual(response.status_code, 302)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import migrations


class ConditionalDeleteModel(migrations.DeleteModel):
    def database_forwards(self, app_label, schema_editor, from_state,
                          to_state):
        try:
            super(ConditionalDeleteModel, self).database_forwards(
                self, app_label, schema_editor, from_state, to_state)
        except:
            """ if it fails, it's totally fine. it just means that the
            table we wanted to delete doesn't exist. so we ignore it.

            it would be nice to catch a more specific exception, but
            different databases raise different ones..."""
            pass


class Migration(migrations.Migration):

    dependencies = [
        ('pagetree', '0001_initial'),
    ]

    operations = [
        ConditionalDeleteModel(
            name='TestBlock',
        ),
    ]
